import { useMemo } from 'react';
import * as THREE from 'three';
import * as turf from '@turf/turf';

export function useWallClassification(walls: Walls) {
    // Find all rooms from the given walls
    const findRooms = () => {
        if (walls.length < 3) return [];

        const pointMap = new Map<string, Point>();
        const connections = new Map<string, string[]>();

        // Build connection graph
        walls.forEach(wall => {
            const [p1, p2] = wall.points;
            if (!pointMap.has(p1.pointUuid)) pointMap.set(p1.pointUuid, p1);
            if (!pointMap.has(p2.pointUuid)) pointMap.set(p2.pointUuid, p2);

            if (!connections.has(p1.pointUuid)) connections.set(p1.pointUuid, []);
            if (!connections.has(p2.pointUuid)) connections.set(p2.pointUuid, []);

            connections.get(p1.pointUuid)?.push(p2.pointUuid);
            connections.get(p2.pointUuid)?.push(p1.pointUuid);
        });
        console.log('connections: ', connections);

        const visited = new Set<string>();
        const rooms: Point[][] = [];

        // Modified DFS to find all possible cycles
        const findAllCycles = (current: string, path: string[]) => {
            visited.add(current);
            path.push(current);

            const neighbors = connections.get(current) || [];
            for (const neighbor of neighbors) {
                if (path.length > 2 && neighbor === path[0]) {
                    // Found a cycle (potential room)
                    const roomPoints = [...path, neighbor].map(uuid => pointMap.get(uuid)!);

                    // Check if this room is valid and not already found
                    if (isValidRoom(roomPoints) && !roomAlreadyExists(rooms, roomPoints)) {
                        rooms.push(roomPoints);
                    }
                    continue;
                }

                if (!path.includes(neighbor)) {
                    findAllCycles(neighbor, [...path]);
                }
            }
        };

        // Start from each point to find all possible cycles
        for (const [pointUuid] of connections) {
            if (!visited.has(pointUuid)) {
                findAllCycles(pointUuid, []);
            }
        }

        return rooms;
    };

    // Helper function to check if room is valid
    const isValidRoom = (roomPoints: Point[]) => {
        if (roomPoints.length < 4) return false;
        const coordinates = roomPoints.map(p => [p.position[0], p.position[2]]);
        const polygon = turf.polygon([coordinates]);
        return turf.booleanValid(polygon);
    };

    // Helper function to check for duplicate rooms
    const roomAlreadyExists = (existingRooms: Point[][], newRoom: Point[]) => {
        const newRoomIds = newRoom.map(p => p.pointUuid).sort().join('-');
        return existingRooms.some(room => {
            const roomIds = room.map(p => p.pointUuid).sort().join('-');
            return roomIds === newRoomIds;
        });
    };

    const rooms = useMemo(() => findRooms(), [walls]);

    // Modified to track all rooms a wall belongs to
    const getWallOrientation = (wall: Wall) => {
        const [p1, p2] = wall.points;
        const orientations: Array<{ isOutsideFacing: boolean }> = [];

        for (const room of rooms) {
            for (let i = 0; i < room.length - 1; i++) {
                const roomP1 = room[i];
                const roomP2 = room[i + 1];

                // Check both directions
                if ((p1.pointUuid === roomP1.pointUuid && p2.pointUuid === roomP2.pointUuid) ||
                    (p1.pointUuid === roomP2.pointUuid && p2.pointUuid === roomP1.pointUuid)) {

                    const roomPoints = room.map(p => new THREE.Vector3(p.position[0], 0, p.position[2]));
                    const centroid = new THREE.Vector3();
                    roomPoints.forEach(p => centroid.add(p));
                    centroid.divideScalar(roomPoints.length);

                    const wallVector = new THREE.Vector3(
                        p2.position[0] - p1.position[0],
                        0,
                        p2.position[2] - p1.position[2]
                    );

                    // Normal depends on wall direction
                    let normal = new THREE.Vector3(-wallVector.z, 0, wallVector.x).normalize();
                    if (p1.pointUuid === roomP2.pointUuid) {
                        normal = new THREE.Vector3(wallVector.z, 0, -wallVector.x).normalize();
                    }

                    const testPoint = new THREE.Vector3(
                        (p1.position[0] + p2.position[0]) / 2 + normal.x,
                        0,
                        (p1.position[2] + p2.position[2]) / 2 + normal.z
                    );

                    const pointInside = turf.booleanPointInPolygon(
                        turf.point([testPoint.x, testPoint.z]),
                        turf.polygon([room.map(p => [p.position[0], p.position[2]])])
                    );

                    orientations.push({
                        isOutsideFacing: !pointInside
                    });
                }
            }
        }

        return {
            isRoomWall: orientations.length > 0,
            orientations // Now tracks all orientations for walls in multiple rooms
        };
    };

    const getWallMaterialSide = (wall: Wall) => {
        const orientation = getWallOrientation(wall);

        if (!orientation.isRoomWall) {
            return { front: 'inside', back: 'inside' }; // Both sides same for segment walls
        }

        // For walls in multiple rooms, we need to determine which side faces which room
        if (orientation.orientations.length === 2) {
            // Wall is between two rooms - one side faces each room's interior
            return {
                front: 'inside',
                back: 'inside'
            };
        } else if (orientation.orientations.length === 1) {
            // Wall is part of only one room (exterior wall)
            return orientation.orientations[0].isOutsideFacing
                ? { front: 'outside', back: 'inside' }
                : { front: 'inside', back: 'outside' };
        }

        // Default case (shouldn't normally happen)
        return { front: 'inside', back: 'inside' };
    };

    // Rest of the functions remain the same
    const getWallType = (wall: Wall) => getWallOrientation(wall).isRoomWall ? 'room' : 'segment';
    const isRoomWall = (wall: Wall) => getWallOrientation(wall).isRoomWall;
    const isSegmentWall = (wall: Wall) => !getWallOrientation(wall).isRoomWall;

    return {
        rooms,
        getWallType,
        isRoomWall,
        isSegmentWall,
        getWallMaterialSide,
        findRooms,
    };
}