import * as THREE from 'three';
import { useMemo } from 'react';
import * as Constants from '../../../../../types/world/worldConstants';

import insideMaterial from '../../../../../assets/textures/floor/wall-tex.png';
import outsideMaterial from '../../../../../assets/textures/floor/factory wall texture.jpg';
import useWallGeometry from './helpers/useWallGeometry';
import { useWallStore } from '../../../../../store/builder/useWallStore';
import { useWallClassification } from './helpers/useWallClassification';

function Wall({ wall }: { readonly wall: Wall }) {
    const { walls } = useWallStore();
    const { getWallMaterialSide, isRoomWall, rooms } = useWallClassification(walls);
    console.log('rooms: ', rooms);
    const materialSide = getWallMaterialSide(wall);
    const [startPoint, endPoint] = wall.points;

    const startX = startPoint.position[0];
    const startZ = startPoint.position[2];
    const endX = endPoint.position[0];
    const endZ = endPoint.position[2];

    const wallLength = Math.sqrt((endX - startX) ** 2 + (endZ - startZ) ** 2);
    const angle = Math.atan2(endZ - startZ, endX - startX);

    const centerX = (startX + endX) / 2;
    const centerZ = (startZ + endZ) / 2;
    const centerY = wall.wallHeight / 2;

    const textureLoader = new THREE.TextureLoader();

    const [insideWallTexture, outsideWallTexture] = useMemo(() => {
        const inside = textureLoader.load(insideMaterial);
        inside.wrapS = inside.wrapT = THREE.RepeatWrapping;
        inside.repeat.set(wallLength / 10, wall.wallHeight / 10);
        inside.colorSpace = THREE.SRGBColorSpace;

        const outside = textureLoader.load(outsideMaterial);
        outside.wrapS = outside.wrapT = THREE.RepeatWrapping;
        outside.repeat.set(wallLength / 10, wall.wallHeight / 10);
        outside.colorSpace = THREE.SRGBColorSpace;

        return [inside, outside];
    }, [wallLength, wall.wallHeight, textureLoader]);

    const materials = useMemo(() => {
        // For segment walls (not in a room), use inside material on both sides
        const frontMaterial = isRoomWall(wall)
            ? (materialSide.front === 'inside' ? insideWallTexture : outsideWallTexture)
            : insideWallTexture;

        const backMaterial = isRoomWall(wall)
            ? (materialSide.back === 'inside' ? insideWallTexture : outsideWallTexture)
            : insideWallTexture;

        return [
            new THREE.MeshStandardMaterial({
                color: Constants.wallConfig.defaultColor,
                side: THREE.DoubleSide,
                map: frontMaterial
            }),
            new THREE.MeshStandardMaterial({
                color: Constants.wallConfig.defaultColor,
                side: THREE.DoubleSide,
                map: backMaterial
            }),
            new THREE.MeshStandardMaterial({ color: Constants.wallConfig.defaultColor, side: THREE.DoubleSide }),  // Left
            new THREE.MeshStandardMaterial({ color: Constants.wallConfig.defaultColor, side: THREE.DoubleSide }),  // Right
            new THREE.MeshStandardMaterial({ color: Constants.wallConfig.defaultColor, side: THREE.DoubleSide }),  // Top
            new THREE.MeshStandardMaterial({ color: Constants.wallConfig.defaultColor, side: THREE.DoubleSide })  // Bottom
        ];
    }, [insideWallTexture, outsideWallTexture, materialSide, isRoomWall, wall]);

    const geometry = useWallGeometry(wallLength, wall.wallHeight, wall.wallThickness);

    return (
        <group
            name={`Wall-${wall.wallUuid}`}
            userData={wall}
            position={[centerX, centerY, centerZ]}
            rotation={[0, -angle, 0]}
        >
            <mesh geometry={geometry}>
                {materials.map((material, index) => (
                    <primitive key={index} object={material} attach={`material-${index}`} />
                ))}
            </mesh>
        </group>
    );
}

export default Wall;